package java.beans;

import com.sun.beans.finder.MethodFinder;
import com.sun.beans.finder.ClassFinder;
import java.lang.reflect.InvocationTargetException;
import sun.reflect.misc.MethodUtil;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import com.sun.beans.finder.ConstructorFinder;
import java.lang.reflect.Array;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.security.AccessController;
import java.security.AccessControlContext;

public class Statement
{
    private static Object[] emptyArray;
    static ExceptionListener defaultExceptionListener;
    private final AccessControlContext acc;
    private final Object target;
    private final String methodName;
    private final Object[] arguments;
    ClassLoader loader;
    
    @ConstructorProperties({ "target", "methodName", "arguments" })
    public Statement(Object target, String methodName, Object[] array) {
        this.acc = AccessController.getContext();
        this.target = target;
        this.methodName = methodName;
        this.arguments = ((array == null) ? Statement.emptyArray : ((Object[])array.clone()));
    }
    
    public Object getTarget() {
        return this.target;
    }
    
    public String getMethodName() {
        return this.methodName;
    }
    
    public Object[] getArguments() {
        return (Object[])this.arguments.clone();
    }
    
    public void execute() throws Exception {
        this.invoke();
    }
    
    Object invoke() throws Exception {
        AccessControlContext acc = this.acc;
        if (acc == null && System.getSecurityManager() != null) {
            throw new SecurityException("AccessControlContext is not set");
        }
        try {
            return AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                @Override
                public Object run() throws Exception {
                    return Statement.access$000(Statement.this);
                }
            }, acc);
        }
        catch (PrivilegedActionException ex) {
            throw ex.getException();
        }
    }
    
    private Object invokeInternal() throws Exception {
        Object target = this.getTarget();
        String methodName = this.getMethodName();
        if (target == null || methodName == null) {
            throw new NullPointerException(((target == null) ? "target" : "methodName") + " should not be null");
        }
        Object[] array = this.getArguments();
        if (array == null) {
            array = Statement.emptyArray;
        }
        if (target == Class.class && methodName.equals("forName")) {
            return ClassFinder.resolveClass((String)array[0], this.loader);
        }
        Class[] args = new Class[array.length];
        for (int i = 0; i < array.length; ++i) {
            args[i] = ((array[i] == null) ? null : array[i].getClass());
        }
        Method method = null;
        if ((target instanceof Class)) {
            if (methodName.equals("new")) {
                methodName = "newInstance";
            }
            if (methodName.equals("newInstance") && ((Class<Class>)target).isArray()) {
                Object instance = Array.newInstance(((Class<Class>)target).getComponentType(), array.length);
                for (int j = 0; j < array.length; ++j) {
                    Array.set(instance, j, array[j]);
                }
                return instance;
            }
            if (methodName.equals("newInstance") && array.length != 0) {
                if (target == Character.class && array.length == 1 && args[0] == String.class) {
                    return new Character(((String)array[0]).charAt(0));
                }
                try {
                    method = ConstructorFinder.findConstructor((Class<Character>)target, args);
                }
                catch (NoSuchMethodException ex) {
                    method = null;
                }
            }
            if (method == null && target != Class.class) {
                method = getMethod((Class<Class>)target, methodName, args);
            }
            if (method == null) {
                method = getMethod(Class.class, methodName, args);
            }
        }
        else if (target.getClass().isArray() && (methodName.equals("set") || methodName.equals("get"))) {
            int intValue = (int)array[0];
            if (methodName.equals("get")) {
                return Array.get(target, intValue);
            }
            Array.set(target, intValue, array[1]);
            return null;
        }
        else {
            method = getMethod(target.getClass(), methodName, args);
        }
        if (method != null) {
            try {
                if ((method instanceof Method)) {
                    return MethodUtil.invoke((Method)method, target, array);
                }
                return ((Constructor)method).newInstance(array);
            }
            catch (IllegalAccessException cause) {
                throw new Exception("Statement cannot invoke: " + methodName + " on " + target.getClass(), cause);
            }
            catch (InvocationTargetException ex2) {
                Throwable targetException = ex2.getTargetException();
                if ((targetException instanceof Exception)) {
                    throw (Exception)targetException;
                }
                throw ex2;
            }
        }
        else {
            throw new NoSuchMethodException(this.toString());
        }
    }
    
    String instanceName(Object o) {
        if (o == null) {
            return "null";
        }
        if (o.getClass() == String.class) {
            return "\"" + (String)o + "\"";
        }
        return NameGenerator.unqualifiedClassName(o.getClass());
    }
    
    @Override
    public String toString() {
        Object target = this.getTarget();
        String methodName = this.getMethodName();
        Object[] array = this.getArguments();
        if (array == null) {
            array = Statement.emptyArray;
        }
        StringBuffer o = new StringBuffer(this.instanceName(target) + "." + methodName + "(");
        for (int length = array.length, i = 0; i < length; ++i) {
            o.append(this.instanceName(array[i]));
            if (i != length - 1) {
                o.append(", ");
            }
        }
        o.append(");");
        return o.toString();
    }
    
    static Method getMethod(Class<?> type, String name, Class<?>... args) {
        try {
            return MethodFinder.findMethod(type, name, args);
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    static Object access$000(Statement statement) throws Exception {
        return statement.invokeInternal();
    }
    
    static {
        Statement.emptyArray = new Object[0];
        Statement.defaultExceptionListener = new ExceptionListener() {
            @Override
            public void exceptionThrown(Exception x) {
                System.err.println(x);
                System.err.println("Continuing ...");
            }
        };
    }
}
